Attribute VB_Name = "modClient"
Option Explicit

Public Const StateNotifyClients As Double = 50 '%
Public Const StateNotifyMin As Long = 2
Public Const StateNotifyMax As Long = 10

Public Enum ClientOnlineState
   cosUnknown = -1
   cosOffline = 0
   cosOnline = 1
   cosAway = 2
   cosLongAway = 3
   cosBreak = 4
   cosBusy = 5
   cosWantChat = 6
   cosInformation = 7
End Enum

Public Enum ClientRelationship
   crNone = 0
   crFriend = 1
   crUnwanted = 2
End Enum

Public Type ClientProperties
   cpUsername As String
   cpRemoteIP As String
   cpLAN_IP As String
   cpOnlineState As ClientOnlineState
   cpTransferForm As dlgFileTransfer
   cpMessageHistory As dlgMessageHistory
   cpStillLoggedIn As Boolean
   cpRelationship As ClientRelationship
   
   cpMessageOk As Boolean
   cpKey As String
   cpRemoteTyping As Boolean
   
End Type

Public LocalUser As ClientProperties
Public LocalStateMessage As String
Public CurrentPassword As String

Public dirIncomingFiles As String
Public dirMessageHistory As String

Public OnlineStateNames() As String

Public Sub Init()
   dirIncomingFiles = modSettings.FileTransferPath
   dirMessageHistory = modSettings.UserPath & "MessageHistory\"
   
   Dim Ret As Long
   
   Ret = MakeSureDirectoryPathExists(dirIncomingFiles)
   Ret = Ret And MakeSureDirectoryPathExists(dirMessageHistory)
   
   If Ret = 0 Then
      Debug.Print "MakeSureDirectoryPathExists() failed in modClient"
   End If

   'set state names'
   ReDim OnlineStateNames(-1 To 6)
   OnlineStateNames(cosUnknown) = "Unbekannt"
   OnlineStateNames(cosOffline) = "Unbekannt"
   OnlineStateNames(cosOnline) = "Unbekannt"
   OnlineStateNames(cosAway) = "Unbekannt"
   OnlineStateNames(cosLongAway) = "Unbekannt"
   OnlineStateNames(cosBreak) = "Unbekannt"
   OnlineStateNames(cosBusy) = "Unbekannt"
   OnlineStateNames(cosWantChat) = "Unbekannt"
   
End Sub

Public Sub ConnectionCancel(NickName As String)
   On Local Error Resume Next
   Dim ChatWindow As dlgChat

   Call dlgMain.CreateConnection.EraseWorkingDialog(NickName)

   For Each ChatWindow In ChatWindows
      If LCase$(ChatWindow.LastChatNick) = LCase$(NickName) And ChatWindow.CountClients = 1 Then
         If Not ChatWindow.Visible Then
            ChatWindows.Remove ChatWindow.ChatId
            Set ChatWindow = Nothing
         End If
      End If
   Next ChatWindow

'   Set ChatWindow = ChatWindows(LCase(NickName))
'   If ChatWindow.Visible = False Then
'      Set ChatWindow = Nothing
'      ChatWindows.Remove LCase(NickName)
'   End If

End Sub

Public Sub SetStateComplete(NewState As ClientOnlineState)
    Dim lSystemBuild As Long
    Const cOldSystem As String = "16"
   'login with new state'
   Call dlgMain.ClientManagement.SetLocalUser(, , NewState)
   Call modConnection.ChangeOnlineState(NewState)
   lSystemBuild = modMain.SystemTray.OSBuild
   
   Select Case NewState
      Case cosOnline
         Call modMain.SystemTray.SetTrayIconData("ActiveVB Messenger (online)", _
            dlgMain.imlBuddyIcons.ListImages("RemoteUser" & IIf(lSystemBuild > 2195, vbNullString, cOldSystem)).ExtractIcon.Handle)
         dlgMain.tlbMain.Buttons("State").Image = "RemoteUser"
      Case cosUnknown
         dlgMain.tlbMain.Buttons("State").Image = "UnknownState"
      Case cosOffline
         dlgMain.tlbMain.Buttons("State").Image = "OfflineUser"
      Case cosAway
         Call modMain.SystemTray.SetTrayIconData("ActiveVB Messenger (abwesend)", _
            dlgMain.imlBuddyIcons.ListImages("StateAway" & IIf(lSystemBuild > 2195, vbNullString, cOldSystem)).ExtractIcon)
         dlgMain.tlbMain.Buttons("State").Image = "StateAway"
      Case cosLongAway
         Call modMain.SystemTray.SetTrayIconData("ActiveVB Messenger (lange abwesend)", _
            dlgMain.imlBuddyIcons.ListImages("StateLongAway" & IIf(lSystemBuild > 2195, vbNullString, cOldSystem)).ExtractIcon)
         dlgMain.tlbMain.Buttons("State").Image = "StateLongAway"
      Case cosBreak
         Call modMain.SystemTray.SetTrayIconData("ActiveVB Messenger (Pause)", _
            dlgMain.imlBuddyIcons.ListImages("StateBreak" & IIf(lSystemBuild > 2195, vbNullString, cOldSystem)).ExtractIcon)
         dlgMain.tlbMain.Buttons("State").Image = "StateBreak"
      Case cosBusy
         Call modMain.SystemTray.SetTrayIconData("ActiveVB Messenger (beschftigt)", _
            dlgMain.imlBuddyIcons.ListImages("StateBusy" & IIf(lSystemBuild > 2195, vbNullString, cOldSystem)).ExtractIcon)
         dlgMain.tlbMain.Buttons("State").Image = "StateBusy"
      Case cosWantChat
         Call modMain.SystemTray.SetTrayIconData("ActiveVB Messenger (chatlustig)", _
            dlgMain.imlBuddyIcons.ListImages("StateWantChat" & IIf(lSystemBuild > 2195, vbNullString, cOldSystem)).ExtractIcon)
         dlgMain.tlbMain.Buttons("State").Image = "StateWantChat"
      Case cosInformation
         Call modMain.SystemTray.SetTrayIconData("ActiveVB Messenger (Information)", _
            dlgMain.imlBuddyIcons.ListImages("StateInformation" & IIf(lSystemBuild > 2195, vbNullString, cOldSystem)).ExtractIcon)
         dlgMain.tlbMain.Buttons("State").Image = "StateInformation"
   End Select

End Sub

