VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CStyleSheetScanner"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------
' Syntax definition of style sheet:
'
' ANY       ::= [\x0-\xFF]
' WS        ::= ' '
'           |   \x8
'           |   \xA
'           |   \xC
' SPACE     ::= WS+
' OBR       ::= '{'
' CBR       ::= '}'
' DEF       ::= ':'
' DELIM     ::= ';'
' LETTER    ::= ['a'-'z''A'-'Z']
' DIGIT     ::= ['0'-'9']
' NUM       ::= DIGIT+ ['.' DIGIT+]
' UL        ::= '_'
' HYPH      ::= '-'
' ID        ::= '#'? LETTER (LETTER | UL | HYPH | DIGIT)*
' QUOTE     ::= '"'
' ESCAPE    ::= '\' ['"' | '\']
' STRING    ::= QUOTE (ANY | ESCAPE)* QUOTE
' CSTART    ::= '/''*'
' CEND      ::= '*''/'
'----------------------------------------------------

Private EOF_    As Boolean
Private Data_   As String
Private DLen_   As Long
Private Pos_    As Long
Private LastPos_ As Long
'----------------------------------------------------

Public Sub StartScan(Data As String)
    Data_ = Data
    DLen_ = LenB(Data_)
    Pos_ = 1
    EOF_ = False
End Sub

Public Function GetNextToken() As TokenType
    Dim State As Long
    
    TokenVal = ""
    LastPos_ = Pos_
    
    Do
        Select Case State
            Case 0
                Select Case GetNextChar()
                    Case "{"
                        State = 1
                    Case "}"
                        State = 2
                    Case ":"
                        State = 3
                    Case ";"
                        State = 4
                    Case "/"
                        State = 5
                    Case " ", vbTab, vbCr, vbLf
                        LastPos_ = Pos_
                    Case "#", "a" To "z", "A" To "Z"
                        State = 8
                    Case "0" To "9"
                        State = 9
                    Case """"
                        State = 12
                    Case Else
                        If Not EOF_ Then
                            ' error: invalid char
                        End If
                End Select
            Case 1
                GetNextToken = OBR
                Exit Function
            Case 2
                GetNextToken = CBR
                Exit Function
            Case 3
                GetNextToken = DEF
                Exit Function
            Case 4
                GetNextToken = DELIM
                Exit Function
            Case 5
                Do
                    Select Case State
                        Case 5
                            Select Case GetNextChar()
                                Case "*"
                                    State = 6
                                Case Else
                                    ' error
                            End Select
                        Case 6
                            Select Case GetNextChar()
                                Case "*"
                                    State = 7
                                Case Else
                            End Select
                        Case 7
                            Select Case GetNextChar()
                                Case "/"
                                    State = 0
                                    LastPos_ = Pos_
                                    Exit Do
                                Case Else
                                    State = 6
                            End Select
                    End Select
                Loop Until EOF_
            Case 8
                Do
                    Select Case State
                        Case 8
                            TokenVal = TokenVal & CurrChar
                            
                            Select Case GetNextChar()
                                Case "a" To "z", "A" To "Z", _
                                     "0" To "9", "_", "-"
                                Case Else
                                    Call UngetChar
                                    GetNextToken = ID
                                    Exit Function
                            End Select
                    End Select
                Loop Until EOF_
                
                GetNextToken = ID
                Exit Function
            Case 9
                Do
                    Select Case State
                        Case 9
                            TokenVal = TokenVal & CurrChar
                            
                            Select Case GetNextChar()
                                Case "0" To "9"
                                Case "."
                                    State = 10
                                Case Else
                                    Call UngetChar
                                    GetNextToken = NUM
                                    Exit Function
                            End Select
                        Case 10
                            TokenVal = TokenVal & CurrChar
                            
                            Select Case GetNextChar()
                                Case "0" To "9"
                                    State = 11
                                Case Else
                                    ' error
                            End Select
                        Case 11
                            TokenVal = TokenVal & CurrChar
                            
                            Select Case GetNextChar()
                                Case "0" To "9"
                                Case Else
                                    Call UngetChar
                                    GetNextToken = NUM
                                    Exit Function
                            End Select
                    End Select
                Loop Until EOF_
                
                GetNextToken = NUM
                Exit Function
            Case 12
                Do
                    Select Case State
                        Case 12
                            Select Case GetNextChar()
                                Case "\"
                                    State = 13
                                Case """"
                                    State = 14
                                Case Else
                                    TokenVal = TokenVal & CurrChar
                            End Select
                        Case 13
                            Select Case GetNextChar()
                                Case "\"
                                    TokenVal = TokenVal & "\"
                                    State = 12
                                Case """"
                                    TokenVal = TokenVal & """"
                                    State = 12
                                Case Else
                                    TokenVal = TokenVal & "\" & CurrChar
                                    State = 12
                            End Select
                        Case 14
                            GetNextToken = STRVAL
                            Exit Function
                    End Select
                Loop Until EOF_
                
                ' error: unclosed string value
        End Select
    Loop Until EOF_
    
    GetNextToken = EOF
End Function

Public Sub Rebound()
    Pos_ = LastPos_
End Sub
'----------------------------------------------------

Private Function GetNextChar() As String
    If Pos_ > DLen_ Then
        EOF_ = True
    Else
        GetNextChar = MidB$(Data_, Pos_, 2)
        Pos_ = Pos_ + 2
    End If
End Function

Private Property Get CurrChar() As String
    CurrChar = MidB$(Data_, Pos_ - 2, 2)
End Property

Private Sub UngetChar()
    Pos_ = Pos_ - 2
End Sub
