Attribute VB_Name = "modHook"
Option Explicit
'**********************************************************************
'* modHook.bas                                                        *
'**********************************************************************
'* Author :  Peter Krner                                             *
'* Date :    27.05.2004                                               *
'* Version : v2.0                                                     *
'**********************************************************************
'* This is the Hooking BackEnd of the Class. Never Use the Module     *
'* directly. Always use the Class!                                    *
'**********************************************************************
'* Annotation: Thanks a lot to Konrad Rudolph. This Class is Based on *
'* his first Release and without him it wouldn't have been so easy to *
'* do this. So: Thank you very much!                                  *
'* For further Information and Help please Ask in the AVB Forum:      *
'*   http://foren.activevb.de/cgi-bin/foren/list.pl?forum=4           *
'**********************************************************************

Private Const GWL_WNDPROC As Long = -4

Private Type Hook
    lWnd As Long
    lPrevWndProc As Long
End Type

Private Type Receiver
    lWnd As Long
    lClassPtr As Long
    laMessages() As Long
    boCatchAll As Boolean
End Type

Private Declare Function CallWindowProc Lib "user32.dll" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hwnd As Long, ByVal msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (nDest As Any, nSrc As Any, ByVal Length As Long)
Private Declare Function SetWindowLong Lib "user32.dll" Alias "SetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long

Private uaHooks() As Hook
Private uaReceiver() As Receiver
Private boInit As Boolean

' Function CreateHook
'  Registers a new Receiver for lWnd. If there's
'  no Hook found for lWnd, a new one is Created.
'
' [IN] lWnd As Long
'  Contains the Window-Handle to be Hooked
' [IN] lClassPtr As Long
'  Contains the Pointer to the Class receiving
'  the Events for this Hook
' [RET] As Long
'  Returns the Internal Index for this Hook.
'  This Index is Used in any Other Call to
'  this Module.
Public Function CreateHook(lWnd As Long, lClassPtr As Long) As Long
    Dim lCounter As Long
    
    ' Init the Structure if necessary
    If Not boInit Then
        ReDim uaHooks(0 To 10)
        ReDim uaReceiver(0 To 10)
        boInit = True
    End If
    
    ' Serch free Slot
    For lCounter = 0 To UBound(uaReceiver)
        If uaReceiver(lCounter).lWnd = 0 Then Exit For
    Next lCounter
    
    ' If there isn't any free Slot enlarge the Structure
    If lCounter > UBound(uaReceiver) Then
        ReDim Preserve uaReceiver(0 To UBound(uaReceiver) + 10)
    End If
    
    ' Put Data into the selected Slot
    uaReceiver(lCounter).lWnd = lWnd
    uaReceiver(lCounter).lClassPtr = lClassPtr
    ReDim uaReceiver(lCounter).laMessages(0 To 10)
    
    ' Return the Reciever-ID
    CreateHook = lCounter
    
    ' Check if there's already a Hook for this Window
    For lCounter = 0 To UBound(uaHooks)
        If uaHooks(lCounter).lWnd = lWnd Then Exit For
    Next lCounter
    
    ' If theres none, search a free Slot
    If lCounter > UBound(uaHooks) Then
        For lCounter = 0 To UBound(uaHooks)
            If uaHooks(lCounter).lWnd = 0 Then Exit For
        Next lCounter
        
        ' If there's furthermore no free Slot
        ' enlarge the Structure
        If lCounter > UBound(uaHooks) Then
            ReDim Preserve uaHooks(0 To UBound(uaHooks) + 10)
        End If
        
        ' Create a new Hook into the first of
        ' the freshly created free Slots
        uaHooks(lCounter).lPrevWndProc = SetWindowLong(lWnd, GWL_WNDPROC, AddressOf WndProc)
        If uaHooks(lCounter).lPrevWndProc = 0 Then Exit Function
        uaHooks(lCounter).lWnd = lWnd
    End If
End Function

' Sub DestroysHook
'  Destroys a Receiver. If there's no other
'  Receiver, using the same Hook found, the
'  Hook for the WindowHandle of this Receiver
'  is Desroyed also
'
' [IN] lID As Long
'  Specifies the ReceiverID to be destroyed
Public Sub DestroyHook(lID As Long)
    Dim lCounter As Long
    
    ' Check if there are Other Receiver using
    ' the same Hook
    For lCounter = 0 To UBound(uaReceiver)
        If uaReceiver(lCounter).lWnd = uaReceiver(lID).lWnd And lID <> lCounter Then Exit For
    Next lCounter
    
    ' If the Hook is only Used from the Receiver
    ' specified by lID, Destroy it
    If lCounter > UBound(uaReceiver) Then
        For lCounter = 0 To UBound(uaHooks)
            If uaHooks(lCounter).lWnd = uaReceiver(lID).lWnd Then Exit For
        Next lCounter
        
        ' Restore the Previous WindowProcedure
        Call SetWindowLong(uaHooks(lCounter).lWnd, GWL_WNDPROC, uaHooks(lCounter).lPrevWndProc)
        
        ' Free the Hooking-Slot
        uaHooks(lCounter).lWnd = 0
        uaHooks(lCounter).lPrevWndProc = 0
    End If
    
    ' Free the Receiver-Slot
    uaReceiver(lID).lClassPtr = 0
    uaReceiver(lID).lWnd = 0
    Erase uaReceiver(lID).laMessages
End Sub

' Sub AddMessage
'  Adds a Message Identified by lMessage
'  to the Receiver Specified by lID
'
' [IN] lID As Long
'  Identifies the Receiver to which the
'  Message should be added
' [IN] lMessage As Long
'  Specifies the Message to be added
Public Sub AddMessage(lID As Long, lMessage As Long)
    Dim lCounter As Long
    
    ' Search for free Message-Slot and simultaneously
    ' for a previously Added Message with the same ID.
    ' If there's one found, exit the sub.
    For lCounter = 0 To UBound(uaReceiver(lID).laMessages)
        If uaReceiver(lID).laMessages(lCounter) = lMessage Then Exit Sub
        If uaReceiver(lID).laMessages(lCounter) = 0 Then Exit For
    Next lCounter
    
    ' If there is no free one, enlarge the Array
    If lCounter > UBound(uaReceiver(lID).laMessages) Then
        ReDim Preserve uaReceiver(lID).laMessages(UBound(uaReceiver(lID).laMessages) + 10)
    End If
    
    ' Put the Data into the Array
    uaReceiver(lID).laMessages(lCounter) = lMessage
End Sub

' Sub RemoveMessage
'  Removes the Message Identified by lMessage
'  from the Receiver (lID)
'
' [IN] lID As Long
'  Identifies the Receiver from which the
'  Message should be removed
' [IN] lMessage As Long
'  Specifies the Message to be removed
Public Sub RemoveMessage(lID As Long, lMessage As Long)
    Dim lCounter As Long
    
    For lCounter = 0 To UBound(uaReceiver(lID).laMessages)
        If uaReceiver(lID).laMessages(lCounter) = lMessage Then Exit For
    Next lCounter
    
    ' If Message isn't found, Exit
    If lCounter > UBound(uaReceiver(lID).laMessages) Then Exit Sub
    
    ' Free the MessageSlot
    uaReceiver(lID).laMessages(lCounter) = 0
End Sub

' Sub SetCatchAll
'  Sets the CatchAll-Flag fpor the Specific Receiver
'
' [IN] lID As Long
'  Identifies the Receiver which Flag
'  should be set
' [IN] boState As Boolean
'  State tho which the Flag should be set
Public Sub SetCatchAll(lID As Long, boState As Boolean)
    uaReceiver(lID).boCatchAll = boState
End Sub

' Function PtrToHook
'  retrieves a clsHook instance from a pointer
'  to do this in an extra procedure means to reduce crashes:
'  if the user halts the execution while in the window proc
'  nothing happens
'  do NEVER halt while in this procedure: program WILL crash!
'  thanks to http://vbaccelerator.com for this trick
' [IN] lpHook As Long
'  Contains a Pointer to a clsHook instance
' [RET] As clsHook
'  returns Instance to the clsHook class, defined by the
'  Pointer.
Private Function PtrToClass(lClassPtr As Long) As clsHook
    Dim tmpObj As clsHook
    
    ' create 'illegal' instance
    Call CopyMemory(tmpObj, lClassPtr, 4)
    ' set 'legal' instance
    Set PtrToClass = tmpObj
    ' destroy 'illegal' instance
    Call CopyMemory(tmpObj, 0&, 4)
End Function

' Function WndProc
'  Window Procedure that handels incoming Messages
' [IN] hWnd As Long
'  Handle to the window
' [IN] uMsg As Long
'  Specifies the message
' [IN] wParam As Long, [IN] lParam As Long
'  Specifies additional message information. The
'  contents of this parameter depend on the value
'  of the uMsg parameter
' [RET] As Long
'  The return value is the result of the message
'  processing and depends on the message sent
'----------------------------------------------------
Private Function WndProc(ByVal hwnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    Dim lCounter As Long
    Dim lCounterTwo As Long
    Dim laClassList() As Long
    Dim oClass As clsHook
    Dim lReturn As Long
    
    ' Init ClassList
    ReDim laClassList(0 To 0)
    
    ' Search for a Receiver for this Window
    For lCounter = 0 To UBound(uaReceiver)
        If uaReceiver(lCounter).lWnd = hwnd Then
            ' If trhe Receiver has the CatchAll-Flag,
            ' don't search
            If Not uaReceiver(lCounter).boCatchAll Then
                ' Search for the Message
                For lCounterTwo = 0 To UBound(uaReceiver(lCounter).laMessages)
                    If uaReceiver(lCounter).laMessages(lCounterTwo) = uMsg Then Exit For
                Next lCounterTwo
            End If
            
            If lCounterTwo <= UBound(uaReceiver(lCounter).laMessages) Or uaReceiver(lCounter).boCatchAll Then
                ' Add to List
                laClassList(UBound(laClassList)) = uaReceiver(lCounter).lClassPtr
                
                ' ReSize Array
                ReDim Preserve laClassList(UBound(laClassList) + 1)
                
                ' Create Class from Pointer
                Set oClass = PtrToClass(uaReceiver(lCounter).lClassPtr)
                
                ' Deliver Before-Event
                lReturn = oClass.RaiseBefore(uMsg, wParam, lParam)
                
                ' Terminate Class
                Set oClass = Nothing
            End If
        End If
    Next lCounter
    
    ' Search Hook
    For lCounter = 0 To UBound(uaHooks)
        If uaHooks(lCounter).lWnd = hwnd Then Exit For
    Next lCounter
    
    If lCounter > UBound(uaHooks) Then Exit Function
    
    ' Forward Message, if not Null
    If lReturn = 0 And CBool(uMsg) Then
    'If CBool(uMsg) Then
        WndProc = CallWindowProc(uaHooks(lCounter).lPrevWndProc, hwnd, uMsg, wParam, lParam)
    ElseIf lReturn <> 0 Then
        WndProc = lReturn
    Else
        WndProc = 0
    End If
    
    ' Die Liste wird Rckwrts durchlaufen weil
    ' ja die weitergabe auch in umgekehrter
    ' Reihenfolge passiert ist.
    For lCounter = UBound(laClassList) - 1 To 0 Step -1
        ' Get Class from Pointer
        Set oClass = PtrToClass(laClassList(lCounter))
        
        ' Deliver Event
        lReturn = oClass.RaiseAfter(uMsg, wParam, lParam)
        
        ' Terminate Class
        Set oClass = Nothing
    Next lCounter
    
    If CBool(uMsg) And lReturn <> 0 Then
        WndProc = lReturn
    End If
    
End Function

Public Function InIDE() As Long
    Static Value As Long
    If Value = 0 Then
        Value = 1
        Debug.Assert InIDE() Or True                          'This line won't exist in the compiled app
        InIDE = Value - 1
    End If
    Value = 0
End Function
